#!/usr/bin/env python
# -*- coding: euc-jp -*-

"""
  \file  CorbaPort.py
  \brief CorbaPort class
  \date  $Date: 2007/09/26 $
  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
 
  Copyright (C) 2006
      Noriaki Ando
      Task-intelligence Research Group,
      Intelligent Systems Research Institute,
      National Institute of
          Advanced Industrial Science and Technology (AIST), Japan
      All rights reserved.
"""

from omniORB import any

import OpenRTM
import RTC, RTC__POA

class CorbaPort(OpenRTM.PortBase):
	
	"""
	\if jp
	\class CorbaPort
	\brief RT ݡͥ CORBA service/consumer  Port

	CorbaPort  RT ݡͥȤˤơ桼 CORBA ֥
	ӥӥ󥷥塼ޤ󶡤 Port Ǥ롣
	<p>
	RT ݡͥȤϡPort 𤷤ƥ桼 CORBA ӥ
	뤳ȤǤ RT Service (Provider) ȸƤ֡
	ޤ¾ RT ݡͥȤΥӥѤ뤿 CORBA ֥
	Υץ졼ۥ󶡤뤳ȤǤ RT Service Consumer ȸƤ֡
	<p>
	CorbaPort Ǥդο Provider  Consumer 뤳ȤǤ
	Port ƱΤ³ݤб Provider  Consumer Ŭڤ˴Ϣդ
	ȤǤ롣
	<p>
	CorbaPort ̾ʲΤ褦Ѥ롣

	<pre>
	self._myServiceProviderPort = OpenRTM.CorbaPort("MyService")
	self._myServiceConsumerPort = OpenRTM.CorbaPort("MyService")

	self._mysvc0 = MyServiceSVC_impl() #  Port 󶡤 Serivce Provider
	self._mycons0 = OpenRTM.CorbaConsumer(interfaceType=_GlobalIDL.MyService) #  Port  Consumer
	# Service Provider  Port Ͽ
	self._myServiceProbiderPort.registerProvider("MyService0", "MyService", self._mysvc0);
	# Service Consumer  Port Ͽ
	self._myServiceConsumerPort.registerConsumer("MyService0", "MyService", self._cons0 );

	# connect Ԥ줿

	self._cons0._ptr().my_service_function() # MyService δؿ򥳡

	</pre>

	Τ褦ˡ󶡤 Service Provider  registerProvider() Ͽ
	뤳Ȥˤꡢ¾ΥݡͥȤѲǽˤ¾
	Ѥ Service Consumer  registerConsumer() Ͽ뤳Ȥˤ
	¾ΥݡͥȤ Service 򥳥ݡͥѲǽˤ뤳Ȥ
	Ǥ롣

	\else
	\class CorbaPort
	\brief RT Conponent CORBA service/consumer Port

	CorbaPort is an implementation of the Port of RT-Component's that provides
	user-defined CORBA Object Service and Consumer.
	<p>
	RT-Component can provide user-defined CORBA serivces, which is called
	RT-Serivce (Provider), through the Ports.
	RT-Component can also provide place-holder, which is called RT-Serivce
	Consumer, to use other RT-Component's service.
	<p>
	The CorbaPort can manage any number of Providers and Consumers, can
	associate Consumers with correspondent Providers when establishing
	connection among Ports.
	<p>
	Usually, CorbaPort is used like the following.

	<pre>
	self._myServiceProviderPort = OpenRTM.CorbaPort("MyService")
	self._myServiceConsumerPort = OpenRTM.CorbaPort("MyService")

	self._mysvc0 = MyServiceSVC_impl() # Serivce Provider that is provided by the Port
	self._mycons0 = OpenRTM.CorbaConsumer(interfaceType=_GlobalIDL.MyService) # Consumer of the Port
	# register Service Provider to the Port
	self._myServiceProbiderPort.registerProvider("MyService0", "MyService", self._mysvc0);
	# register Service Consumer to the Port
	self._myServiceConsumerPort.registerConsumer("MyService0", "MyService", self._cons0 );

	# after connect established

	self._cons0._ptr().my_service_function() # call a MyService's function
	</pre>

	Registering Service Provider by registerProvider(), it can be used from
	other RT-Components.
	Registering Service Consumer by registerConsumer(), other RT-Component's
	services can be used through the consumer object.

	\endif
	"""


	def __init__(self, name):
		"""
		\if jp
		\brief 󥹥ȥ饯
		\param name Port ̾
		\else
		\brief Constructor
		\param name The name of Port 
		\endif
		"""
		OpenRTM.PortBase.__init__(self, name)
		self.addProperty("port.port_type", "CorbaPort")
		self._providers = []
		self._consumers = []


	def __del__(self):
		"""
		\if jp
		\brief ǥȥ饯
		\else
		\brief Destructor
		\endif
		"""
		pass


	def registerProvider(self, instance_name, type_name, provider):
		"""
		\if jp
		\brief Provider Ͽ
		
		 Port ˤ󶡤ХȤ򤳤 Port ФϿ롣
		ХȤϡͿ instance_name, type_name 
		ХȼȤΥ󥹥̾ӥ̾ȤơХȤ
		Ϣդ롣
		\param instance_name ХȤΥ󥹥̾
		\param type_name ХȤΥ̾
		\param provider CORBA Х
		\return Ʊ̾ instance_name ϿƤ false ֤
		\else
		\brief Register provider
		
		This operation registers a servant, which is provided in this Port,
		to the Port. The servant is associated with "instance_name" and
		"type_name" as the instance name of the servant and as the type name
		of the servant.
		\param instance_name ХȤΥ󥹥̾
		\param type_name ХȤΥ̾
		\param provider CORBA Х
		\return Ʊ̾ instance_name ϿƤ false ֤
		\endif
		"""
		if not self.appendInterface(instance_name, type_name, RTC.PROVIDED):
			return False

		oid = self._default_POA().activate_object(provider)
		obj = self._default_POA().id_to_reference(oid)

		key = "port"
		key = key + "." + str(type_name) + "." + str(instance_name)

		OpenRTM.CORBA_SeqUtil.push_back(self._providers,
										OpenRTM.NVUtil.newNV(key, obj))

		return True


	def registerConsumer(self, instance_name, type_name, consumer):
		"""
		\if jp
		\brief Consumer Ͽ
		
		 Port ׵᤹륵ӥΥץ졼ۥǤ륳󥷥塼
		(Consumer) Ͽ롣
		Consumer Ϣդ륵ӥΥ󥹥̾ӥ̾Ȥơ
		 instance_name, type_name  Consumer ȤͿ뤳Ȥˤꡢ
		Ǥ餬Ϣդ롣
		Port ֤³ (connect)  ˤϡƱ instance_name, type_name 
		ӥ¾ Port  (Provide) Ƥ硢Υӥ
		֥ȻȤưŪ Consumer ˥åȤ롣
		\param instance_name Consumer ׵᤹륵ӥΥ󥹥̾
		\param type_name Consumer ׵᤹륵ӥΥ̾
		\param consumer CORBA ӥ󥷥塼
		\return Ʊ̾ instance_name ϿƤ false ֤
		\else
		\brief Register consumer
		
		This operation registers a consumer, which requiers a service,
		to the other Port. The consumer is associated with "instance_name" and
		"type_name" as the instance name of the service and as the type name
		of the service that is required.
		\param instance_name An instance name of the service required
		\param type_name An type name of the service required
		\param consumer CORBA service consumer
		\return False would be returned if the same instance_name is registered
		\endif
		"""
		if not self.appendInterface(instance_name, type_name, RTC.REQUIRED):
			return False

		cons = self.Consumer(instance_name, type_name, consumer)
		self._consumers.append(cons)

		return True


	def publishInterfaces(self, connector_profile):
		"""
		\if jp
		\brief Interface 
		
		 Portͭ Provider ˴ؤ ConnectorProfile::properties
		롣
		ϡNVList name  value ȤưʲΤΤǼ롣
		
		- port.<type_name>.<instance_name>: <CORBA::Object_ptr>
		
		ǡ
		- <type_name>: PortInterfaceProfile::type_name
		- <instance_name>: PortInterfaceProfile::instance_name
		Ǥ롣
		ConnectorProfile::properties Ǥϡ .(ɥå)ɽǡ
		NameValue ΥȤƤ롣äơ
		
		<pre>
		PortInterfaceProfile
		{
        instance_name = "PA10_0";
        type_name     = "Manipulator";
        polarity      = PROVIDED;
		}
		</pre>
		
		ʤС
		
		<pre>
		NameValue = { "port.Manipulator.PA10_0": <Object reference> }
		</pre>
		
		Ȥäͤ ConnectorProfile::properties ˳Ǽ졢¾ΥݡȤФ
		ã롣¾ Port ǤΥ󥿡եѤ Consumer 
		¸ߤСConnectorProfile 餳Υ饪֥ȥե󥹤
		餫ηǻѤ롣
		\else
		\brief Publish interface information
		\endif
		"""
		OpenRTM.CORBA_SeqUtil.push_back_list(connector_profile.properties,
											 self._providers)
		return RTC.RTC_OK


	def subscribeInterfaces(self, connector_profile):
		"""
		\if jp

		\brief Interface ³

		 Portͭ Consumer Ŭ礹 Provider ˴ؤ
		ConnectorProfile::properties Ф Consumer ˥֥Ȼ
		򥻥åȤ롣

		Consumer 
		<pre>
			PortInterfaceProfile
			{
				instance_name = "PA10_0";
				type_name     = "Manipulator";
				polarity      = REQUIRED;
			}
		</pre>
		ȤϿƤС¾ Port 
		<pre>
			PortInterfaceProfile
			{
				instance_name = "PA10_0";
				type_name     = "Manipulator";
				polarity      = PROVIDED;
			}
		</pre> 
		ȤϿƤ Serivce Provider Υ֥ȻȤõ
		Consumer ˥åȤ롣
		ºݤˤϡConnectorProfile::properties 
		<pre>
		NameValue = { "port.Manipulator.PA10_0": <Object reference> }
		</pre>
		ȤϿƤ NameValue õΥ֥ȻȤ
		Consumer ˥åȤ롣

		\else

		\brief Subscribe interfaces

		\endif
		"""
		nv = connector_profile.properties
		OpenRTM.CORBA_SeqUtil.for_each(nv, self.subscribe(self._consumers))
		return RTC.RTC_OK


 	def unsubscribeInterfaces(self, connector_profile):
		"""
		\if jp
		\brief Interface ؤ³
		
		Ϳ줿 ConnectorProfile ˴Ϣ Consumer ˥åȤ줿
		٤Ƥ Object ³롣
		\else
		\brief Unsubscribe interfaces
		\endif
		"""
		nv = connector_profile.properties

		OpenRTM.CORBA_SeqUtil.for_each(nv, self.unsubscribe(self._consumers))


 	class Consumer:
		
		"""
		\if jp
		\brief Consumer ξǼ빽¤
		\else
		\brief Consumer inforamtion struct
		\endif
		"""

		def __init__(self, _instance_name, _type_name, _cons, _consumer=None):
			if _consumer:
				self.name = _consumer.name
				self.consumer = _consumer.consumer
				return
			
			self.name = "port."+str(_type_name)+"."+str(_instance_name)
			self.consumer = _cons


	class subscribe:
		
		"""
		\if jp
		\brief ConnectorProfile  Consuemr Ӥ򤷥֥ȻȤ
		åȤ뤿 Functor
		\else
		\brief Subscription mutching functor for Consumer
		\endif
		"""
		def __init__(self, cons):
			self._cons = cons
			self._len  = len(cons)

		def __call__(self, nv):
			for i in range(self._len):
				name_ = nv.name
				if self._cons[i].name == name_:
					try:
						obj = any.from_any(nv.value, keep_structs=True)
						self._cons[i].consumer.setObject(obj)
					except:
						pass


	class unsubscribe:
		
		"""
		\if jp
		\brief Consumer Υ֥Ȥ뤿 Functor
		\else
		\brief Unsubscription functor for Consumer
		\endif
		"""
		def __init__(self, cons):
			self._cons = cons
			self._len  = len(cons)

		def __call__(self, nv):
			for i in range(self._len):
				name_ = nv.name
				if self._cons[i].name == name_:
					self._cons[i].consumer.releaseObject()
