#!/usr/bin/env python
# -*- coding: euc-jp -*-

"""
  \file DataOutPort.py
  \brief Base class of OutPort
  \date $Date: 2007/09/20 $
  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
 
  Copyright (C) 2006
      Noriaki Ando
      Task-intelligence Research Group,
      Intelligent Systems Research Institute,
      National Institute of
          Advanced Industrial Science and Technology (AIST), Japan
      All rights reserved.
"""


import OpenRTM
import RTC, RTC__POA


class DataOutPort(OpenRTM.PortBase):
	
	"""
	\if jp
	\class DataOutPort
	\brief InPort  Port
	\else
	\class DataOutPort
	\brief InPort abstruct class
	\endif
	"""


	def __init__(self, name, outport):
		"""
		\if jp
		\brief 󥹥ȥ饯
		\else
		\brief Constructor
		\endif
		"""
		OpenRTM.PortBase.__init__(self, name)
		self._outport = outport
		# PortProfile::properties 
		self.addProperty("port.port_type", "DataOutPort")
		self._providers = []
		self._providers.append(OpenRTM.OutPortCorbaProvider(outport))
		self._providers[-1].publishInterfaceProfile(self._profile.properties)
		self._consumers = []
		self._consumers.append(OpenRTM.InPortCorbaConsumer(outport))
		self._pf = OpenRTM.PublisherFactory()


	def __del__(self):
		"""
		\if jp
		\brief ǥȥ饯
		\else
		\brief Destructor
		\endif
		"""
		pass

    
	"""
      \if jp
      \brief [CORBA interface] Port ³Ԥ
     
      OutPort  InPort Ȥ³Ԥ
     
      OutPort ¦ connect() ǤϰʲΥ󥹤ǽԤ롣
     
      1. OutPort ˴Ϣ connector ӥå
     
      2. InPort˴Ϣ connector μ
       - ConnectorProfile::properties["dataport.corba_any.inport_ref"]
         OutPortAny Υ֥ȥե󥹤ꤵƤ硢
         ե󥹤Consumer֥Ȥ˥åȤ롣
         ե󥹤åȤƤʤ̵뤷Ʒ³
         (OutPortconnect() ƤӽФΥȥݥȤξϡ
         InPortΥ֥ȥե󥹤ϥåȤƤʤϤǤ롣)
      3. PortBase::connect() 򥳡
         Port³δܽԤ롣
      4. 嵭2.InPortΥե󥹤ǤʤСInPort
         Ϣ connector 롣
     
      5. ConnectorProfile::properties Ϳ줿󤫤顢
         OutPort¦νԤ
     
      - [dataport.interface_type]
      -- CORBA_Any ξ: 
         InPortAny ̤ƥǡ򴹤롣
         ConnectorProfile::properties["dataport.corba_any.inport_ref"]
         InPortAny Υ֥ȥե󥹤򥻥åȤ롣
      -- RawTCP ξ: Raw TCP socket ̤ƥǡ򴹤롣
         ConnectorProfile::properties["dataport.raw_tcp.server_addr"]
         InPort¦ΥХɥ쥹򥻥åȤ롣
     
      - [dataport.dataflow_type]
      -- Pushξ: Subscriber롣SubscriberΥפϡ
         dataport.subscription_type ꤵƤ롣
      -- Pullξ: InPort¦ǡPullǼ뤿ᡢ
         ä˲⤹ɬפ̵
     
      - [dataport.subscription_type]
      -- Onceξ: SubscriberOnce롣
      -- Newξ: SubscriberNew롣
      -- Periodicξ: SubscriberPeriodic롣
     
      - [dataport.push_interval]
      -- dataport.subscription_type=Periodicξꤹ롣
     
      6. 嵭νΤĤǤ⥨顼ǤС顼꥿󤹤롣
         ˽Ԥ줿RTC::RTC_OKǥ꥿󤹤롣
       
      \else
      \brief [CORBA interface] Connect the Port
      \endif
	"""


	def publishInterfaces(self, connector_profile):
		"""
		\if jp

		\brief Interface 

		Υڥ졼ϡnotify_connect() 󥹤λϤ˥
		ؿǤ롣
		notify_connect() Ǥϡ

		- publishInterfaces()
		- connectNext()
		- subscribeInterfaces()
		- updateConnectorProfile()

		ν˴ؿ뤵³Ԥ롣
		<br>
		Υڥ졼ϡ connector_id ФƤ³
		¸ connector_id ФƤϹŬڤ˹Ԥɬפ롣

		\param connector_profile ³˴ؤץե
		\return ReturnCode_t Υ꥿󥳡

		\else

		\brief Publish interface information

		This operation is method that would be called at the
		beginning of the notify_connect() process sequence.
		In the notify_connect(), the following methods would be called in order.

		- publishInterfaces()
		- connectNext()
		- subscribeInterfaces()
		- updateConnectorProfile() 

		This operation should create the new connection for the new
		connector_id, and should update the connection for the existing
		connection_id.

		\param connector_profile The connection profile information
		\return The return code of ReturnCode_t type.

		\endif
		"""
		publish = self.publish(connector_profile.properties)
		for provider in self._providers:
			publish(provider)
		return RTC.RTC_OK


	def subscribeInterfaces(self, connector_profile):
		"""
		\if jp

		\brief Interface 

		Υڥ졼ϡnotify_connect() 󥹤֤˥
		ؿǤ롣
		notify_connect() Ǥϡ

		- publishInterfaces()
		- connectNext()
		- subscribeInterfaces()
		- updateConnectorProfile()

		ν˴ؿ뤵³Ԥ롣

		\param connector_profile ³˴ؤץե
		\return ReturnCode_t Υ꥿󥳡

		\else

		\brief Publish interface information

		This operation is method that would be called at the
		mid-flow of the notify_connect() process sequence.
		In the notify_connect(), the following methods would be called in order.

		- publishInterfaces()
		- connectNext()
		- subscribeInterfaces()
		- updateConnectorProfile()

		\param connector_profile The connection profile information
		\return The return code of ReturnCode_t type.

		\endif
		"""
		subscribe = self.subscribe(prof=connector_profile)
		for consumer in self._consumers:
			subscribe(consumer)


		if not subscribe._consumer:
			return RTC.RTC_OK

    
		# Publisher
		prop = OpenRTM.NVUtil.toProperties(connector_profile.properties)
		publisher = self._pf.create(subscribe._consumer.clone(), prop)

		# PublisherOutPort˥å
		self._outport.attach(connector_profile.connector_id, publisher)

		return RTC.RTC_OK


	def unsubscribeInterfaces(self, connector_profile):
		"""
		\if jp

		\brief Interface ³

		Υڥ졼ϡnotify_disconnect() 󥹤ν˥
		ؿǤ롣
		notify_disconnect() Ǥϡ
		- disconnectNext()
		- unsubscribeInterfaces()
		- eraseConnectorProfile()
		ν protected ؿ뤵³Ԥ롣

		\param connector_profile ³˴ؤץե

		\else

		\brief Disconnect interface connection

		This operation is method that would be called at the
		end of the notify_disconnect() process sequence.
		In the notify_disconnect(), the following methods would be called.
		- disconnectNext()
		- unsubscribeInterfaces()
		- eraseConnectorProfile() 

		\param connector_profile The connection profile information

		\endif
		"""
		publisher = self._outport.detach(connector_profile.connector_id)
		self._pf.destroy(publisher)
		return


	class publish:
		
		def __init__(self, prop):
			self._prop = prop

		def __call__(self, provider):
			provider.publishInterface(self._prop)


	class unsubscribe:
		
		def __init__(self, prop):
			self._prop = prop

		def __call__(self, consumer):
			consumer.unsubscribeInterface(self._prop)


	class subscribe:
		
		def __init__(self, prof=None, subs=None):
			if prof and not subs:
				self._prof = prof
				self._consumer = None
			elif not prof and subs:
				self._prof = subs._prof
				self._consumer = subs._consumer
			else:
				print "DataOutPort.subscribe: Invalid parameter."


		def __call__(self, cons):
			if cons.subscribeInterface(self._prof.properties):
				self._consumer = cons
